﻿//----------------------------------------------------------
// DS102/DS112コントローラ制御用サンプルプログラム
//
// Copyright 2023 SURUGA SEIKI Co.,Ltd. All rights reserved.
//----------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace DS102CSharpSample
{
    public partial class Form1 : Form
    {
        private string AxisNo = "1";        //軸番号
        private string Direction;           //駆動方向設定 (-(CCW)、+(CW))
        private int Mode = 0;               //駆動方法 (0:連続駆動、1:ステップ駆動、2:原点復帰)

        ///<summary>
        ///初期設定
        ///</summary>
        public Form1()
        {
            InitializeComponent();

            //ソフトウェアバージョンを表示する
            System.Reflection.Assembly asm = System.Reflection.Assembly.GetExecutingAssembly();
            Version ver = asm.GetName().Version;
            Version softwareVersion = new Version(ver.Major, ver.Minor, ver.Build);

            this.Text = "DS102/DS112コントローラ・サンプルプログラム Ver." + softwareVersion;

            ComboBoxOrg.SelectedIndex = 0;
            ComboBoxCommPort.SelectedIndex = 0;
            ComboBoxBaudrate.SelectedIndex = 0;
            TextBoxPosition.Text = "0";
        }

        ///<summary>
        ///接続ボタンを押した時の処理
        ///</summary>
        private void ButtonConnect_Click(object sender, EventArgs e)
        {
            //通信ポートを設定する
            CommPortOpen();
        }

        ///<summary>
        ///通信ポートを設定する
        ///</summary>
        private void CommPortOpen()
        {
            if (SerialPort != null)
                SerialPort.Close();

            SerialPort.PortName = ComboBoxCommPort.Text;                //COM番号
            SerialPort.BaudRate = int.Parse(ComboBoxBaudrate.Text);     //ボーレート
            SerialPort.NewLine = "\r";                                  //デリミタコード
            SerialPort.ReadTimeout = 2000;                              //読取りタイムアウト時間
            SerialPort.WriteTimeout = 2000;                             //書込みタイムアウト時間

            try
            {
                //通信ポート接続
                SerialPort.Open();
            }
            catch (Exception ex)
            {
                LabelState.Text = "接続エラー発生";
                MessageBox.Show(ex.Message);
                return;
            }

            System.Threading.Thread.Sleep(100);

            //---------------------------------------------------------
            // ID要求 DS102/DS112コントローラ接続確認
            //---------------------------------------------------------
            string data = SendReceiveString("*IDN?");

            if (!string.IsNullOrEmpty(data) ? data.Contains("SURUGA,DS1") : false)
            {
                //---------------------------------------------------------
                //バージョン要求
                //---------------------------------------------------------
                data = SendReceiveString("DS102VER?");
                LabelFirmware.Text = data;
                
                //---------------------------------------------------------
                //制御軸数ステータス
                //---------------------------------------------------------
                data = SendReceiveString("CONTA?");

                ButtonAxisX.Enabled = false;
                ButtonAxisY.Enabled = false;
                ButtonAxisZ.Enabled = false;
                ButtonAxisU.Enabled = false;
                ButtonAxisV.Enabled = false;
                ButtonAxisW.Enabled = false;

                //取得した軸数 2(X、Y)
                if (int.Parse(data) == 2)
                {
                    //---------------------------------------------------------
                    // 表示単位設定・速度テーブル設定
                    //---------------------------------------------------------
                    for (int axNo = 1; axNo <= 2; axNo++)
                    {
                        SendString("AXI" + axNo.ToString() + ":UNIT 0:SELSP 0");
                        System.Threading.Thread.Sleep(100);
                    }

                    ButtonAxisX.Enabled = true;
                    ButtonAxisY.Enabled = true;
                }
                //取得した軸数 4(X～U)
                else if (int.Parse(data) == 4)
                {
                    //---------------------------------------------------------
                    // 表示単位設定・速度テーブル設定
                    //---------------------------------------------------------
                    for (int axNo = 1; axNo <= 4; axNo++)
                    {
                        SendString("AXI" + axNo.ToString() + ":UNIT 0:SELSP 0");
                        System.Threading.Thread.Sleep(100);
                    }

                    ButtonAxisX.Enabled = true;
                    ButtonAxisY.Enabled = true;
                    ButtonAxisZ.Enabled = true;
                    ButtonAxisU.Enabled = true;
                }
                //取得した軸数 6(X～W)
                else
                {
                    //---------------------------------------------------------
                    // 表示単位設定・速度テーブル設定
                    //---------------------------------------------------------
                    for (int axNo = 1; axNo <= 6; axNo++)
                    {
                        SendString("AXI" + axNo.ToString() + ":UNIT 0:SELSP 0");
                        System.Threading.Thread.Sleep(100);
                    }

                    ButtonAxisX.Enabled = true;
                    ButtonAxisY.Enabled = true;
                    ButtonAxisZ.Enabled = true;
                    ButtonAxisU.Enabled = true;
                    ButtonAxisV.Enabled = true;
                    ButtonAxisW.Enabled = true;
                }

                //ステータスを更新する
                UpdateStatus();
            }
            else
            {
                LabelState.Text = "受信エラー発生";
                MessageBox.Show(ComboBoxCommPort.Text + "にDS102/DS112コントローラが接続されていません");
            }
        }

        ///<summary>
        ///切断ボタンを押した時の処理
        ///</summary>
        private void ButtonDisconnect_Click(object sender, EventArgs e)
        {
            if (SerialPort != null)
                SerialPort.Close();

            ButtonAxisX.Enabled = true;
            ButtonAxisY.Enabled = true;
            ButtonAxisZ.Enabled = true;
            ButtonAxisU.Enabled = true;
            ButtonAxisV.Enabled = true;
            ButtonAxisW.Enabled = true;

            LabelState.Text = "未接続";
        }

        ///<summary>
        ///X軸選択ボタンを押した時の処理
        ///</summary>
        private void ButtonAxisX_CheckedChanged(object sender, EventArgs e)
        {
            AxisNo = "1";           //使用軸をX軸に設定
            UpdateStatus();         //ステータスを更新する
        }

        ///<summary>
        ///Y軸選択ボタンを押した時の処理
        ///</summary>
        private void ButtonAxisY_CheckedChanged(object sender, EventArgs e)
        {
            AxisNo = "2";           //使用軸をY軸に設定
            UpdateStatus();         //ステータスを更新する
        }

        ///<summary>
        ///Z軸選択ボタンを押した時の処理
        ///</summary>
        private void ButtonAxisZ_CheckedChanged(object sender, EventArgs e)
        {
            AxisNo = "3";           //使用軸をZ軸に設定
            UpdateStatus();         //ステータスを更新する
        }

        ///<summary>
        ///U軸選択ボタンを押した時の処理
        ///</summary>
        private void ButtonAxisU_CheckedChanged(object sender, EventArgs e)
        {
            AxisNo = "4";           //使用軸をU軸に設定
            UpdateStatus();         //ステータスを更新する
        }

        ///<summary>
        ///V軸選択ボタンを押した時の処理
        ///</summary>
        private void ButtonAxisV_CheckedChanged(object sender, EventArgs e)
        {
            AxisNo = "5";           //使用軸をV軸に設定
            UpdateStatus();         //ステータスを更新する
        }

        ///<summary>
        ///W軸選択ボタンを押した時の処理
        ///</summary>
        private void ButtonAxisW_CheckedChanged(object sender, EventArgs e)
        {
            AxisNo = "6";           //使用軸をW軸に設定
            UpdateStatus();         //ステータスを更新する
        }

        ///<summary>
        ///連続駆動を押した時の処理
        ///</summary>
        private void RadioButtonContinueMode_CheckedChanged(object sender, EventArgs e)
        {
            ButtonCCW.Text = "- (CCW)";
            ButtonCW.Text = "+ (CW)";
            Mode = 0;
        }

        ///<summary>
        ///ステップ駆動を押した時の処理
        ///</summary>
        private void RadioButtonStepMode_CheckedChanged(object sender, EventArgs e)
        {
            ButtonCCW.Text = "- (CCW)";
            ButtonCW.Text = "+ (CW)";
            Mode = 1;
        }

        ///<summary>
        ///原点復帰を押した時の処理
        ///</summary>
        private void RadioButtonOrgMode_CheckedChanged(object sender, EventArgs e)
        {
            ButtonCCW.Text = "原点復帰開始";
            ButtonCW.Text = "原点復帰開始";
            Mode = 2;
        }

        ///<summary>
        ///停止ボタンを押した時の処理
        ///</summary>
        private void ButtonStop_Click(object sender, EventArgs e)
        {
            //---------------------------------------------------------
            // 停止
            //---------------------------------------------------------
            SendString("STOP 0");
        }

        ///<summary>
        ///CCWボタンを押した時の処理
        ///</summary>
        private void ButtonCCW_MouseDown(object sender, MouseEventArgs e)
        {
            Direction = "CCW";              //モータの駆動方向をCCWに設定する
            MoveStage();                    //駆動を開始する
        }

        ///<summary>
        ///CCWボタンを離した時の処理
        ///</summary>
        private void ButtonCCW_MouseUp(object sender, MouseEventArgs e)
        {
            //駆動方法が連続駆動の場合
            if (Mode == 0)
                //---------------------------------------------------------
                // 停止
                //---------------------------------------------------------
                SendString("STOP 0");       //軸を停止する
        }

        ///<summary>
        ///CWボタンを押した時の処理
        ///</summary>
        private void ButtonCW_MouseDown(object sender, MouseEventArgs e)
        {
            Direction = "CW";               //モータの駆動方向をCWに設定する
            MoveStage();                    //駆動を開始する
        }

        ///<summary>
        ///CWボタンを離した時の処理
        ///</summary>
        private void ButtonCW_MouseUp(object sender, MouseEventArgs e)
        {
            //駆動方法が連続駆動の場合
            if (Mode == 0)
                //---------------------------------------------------------
                // 停止
                //---------------------------------------------------------
                SendString("STOP 0");       //軸を停止する
        }

        ///<summary>
        ///ステージを駆動する
        ///</summary>
        private void MoveStage()
        {
            //駆動方法
            switch (Mode)
            {
                //連続駆動
                case 0:
                    //モータの駆動方向がCWの場合
                    if (Direction == "CW")
                        //---------------------------------------------------------
                        // 連続駆動 CW方向
                        //---------------------------------------------------------
                        SendString("AXI" + AxisNo + ":L0 " + TextBoxLSpeed.Text +
                                   ":R0 " + TextBoxRate.Text + ":S0 " + TextBoxSRate.Text +
                                   ":F0 " + TextBoxSpeed.Text + ":GO CWJ");
                    //モータの駆動方向がCCWの場合
                    else
                        //---------------------------------------------------------
                        // 連続駆動 CCW方向
                        //---------------------------------------------------------
                        SendString("AXI" + AxisNo + ":L0 " + TextBoxLSpeed.Text +
                                   ":R0 " + TextBoxRate.Text + ":S0 " + TextBoxSRate.Text +
                                   ":F0 " + TextBoxSpeed.Text + ":GO CCWJ");
                    break;
                //ステップ駆動
                case 1:
                    //---------------------------------------------------------
                    // ステップ駆動
                    //---------------------------------------------------------
                    SendString("AXI" + AxisNo + ":L0 " + TextBoxLSpeed.Text +
                              ":R0 " + TextBoxRate.Text + ":S0 " + TextBoxSRate.Text +
                              ":F0 " + TextBoxSpeed.Text + ":PULS " + TextBoxStep.Text + ":GO " + Direction);
                    break;
                //原点復帰
                case 2:
                    //原点復帰方式を取得する
                    string orgMode = ComboBoxOrg.SelectedIndex.ToString();

                    //原点復帰方式を変更する
                    //---------------------------------------------------------
                    // メモリスイッチ0設定
                    //---------------------------------------------------------
                    SendString("AXI" + AxisNo + ":MEMSW0 " + orgMode);

                    System.Threading.Thread.Sleep(100);

                    //---------------------------------------------------------
                    // 原点復帰
                    //---------------------------------------------------------
                    SendString("AXI" + AxisNo + ":L0 " + TextBoxLSpeed.Text +
                               ":R0 " + TextBoxRate.Text + ":S0 " + TextBoxSRate.Text +
                               ":F0 " + TextBoxSpeed.Text + ":GO ORG");
                    break;
            }

            //ステータスを更新する
            Timer.Start();
        }

        ///<summary>
        ///画面更新処理
        ///</summary>
        private void Timer_Tick(object sender, EventArgs e)
        {
            if (SerialPort.IsOpen)
                //ステータスを更新する
                UpdateStatus();
        }

        ///<summary>
        ///ステータスを更新する
        ///</summary>
        private void UpdateStatus()
        {
            string data;

            //---------------------------------------------------------
            //ステータス3要求
            //---------------------------------------------------------
            data = SendReceiveString("AXI" + AxisNo + ":SB3?");

            //数値に変換できなければ終了する
            if (!int.TryParse(data, out int result))
            {
                Timer.Stop();
                return;
            }

            //軸選択不可の場合
            if ((int.Parse(data) & 0x01) != 0x01)
            {
                LabelState.Text = "軸選択不可能";
                Timer.Stop();
            }
            else
            {
                //---------------------------------------------------------
                //ステータス1要求
                //---------------------------------------------------------
                data = SendReceiveString("AXI" + AxisNo + ":SB1?");
                
                if ((int.Parse(data) & 0x40) == 0x40)
                {
                    LabelState.Text = "動作中";
                }
                else if ((int.Parse(data) & 0x10) == 0x10)
                {
                    LabelState.Text = "原点検出";
                    Timer.Stop();
                }               
                else if ((int.Parse(data) & 0x02) == 0x02 || (int.Parse(data) & 0x04) == 0x04)
                {
                    //リミット検出
                    //---------------------------------------------------------
                    //ステータス2要求(リミットステータス確認)
                    //---------------------------------------------------------
                    data = SendReceiveString("AXI" + AxisNo + ":SB2?");

                    if ((int.Parse(data) & 0x03) == 0x03)
                    {
                        LabelState.Text = "ステージ未接続";
                    }
                    else if ((int.Parse(data) & 0x01) == 0x01)
                    {
                        LabelState.Text = "CWリミット検出";
                    }
                    else if ((int.Parse(data) & 0x02) == 0x02)
                    {
                        LabelState.Text = "CCWリミット検出";
                    }
                    else if ((int.Parse(data) & 0x04) == 0x04)
                    {
                        LabelState.Text = "CWソフトリミット検出";
                    }
                    else if ((int.Parse(data) & 0x08) == 0x08)
                    {
                        LabelState.Text = "CCWソフトリミット検出";
                    }

                    Timer.Stop();
                }
                else
                {
                    LabelState.Text = "停止";
                    Timer.Stop();
                }

                //---------------------------------------------------------
                //現在位置要求
                //---------------------------------------------------------
                data = SendReceiveString("AXI" + AxisNo + ":POS?");
                TextBoxPosition.Text = data;
            }
        }

        ///<summary>
        ///送信
        ///</summary>
        ///<param name="cmd">送信文字列</param>
        public void SendString(string cmd)
        {
            try
            {
                if (SerialPort.IsOpen)
                    SerialPort.WriteLine(cmd);
            }
            catch (Exception ex)
            {
                LabelState.Text = "送信エラー発生";
                MessageBox.Show(ex.Message);
            }
        }

        ///<summary>
        ///受信
        ///</summary>
        ///<returns>受信文字列</returns>
        public string ReceiveString()
        {
            DateTime timeoutTime = DateTime.Now.AddMilliseconds(2000);      //タイムアウト時間
            string rcvData = null;                                          //受信データ

            //終端文字まで受信処理を行う
            while (true)
            {
                try
                {
                    int len = SerialPort.BytesToRead;

                    //タイムアウトまで受信を試みる
                    while (len < 1)
                    {
                        System.Threading.Thread.Sleep(50);

                        if (DateTime.Now > timeoutTime)
                        {
                            LabelState.Text = "通信タイムアウト発生";
                            return null;
                        }

                        len = SerialPort.BytesToRead;
                    }

                    int readBytes = 0;
                    byte[] inByte = new byte[len];

                    //受信バッファ内データ読出
                    SerialPort.Read(inByte, readBytes, len);
                    //受信データを文字列に格納
                    string rcvDataTemp = Encoding.ASCII.GetString(inByte);

                    rcvData += rcvDataTemp;

                    //終端文字列の位置確認
                    int index = rcvData.IndexOf(SerialPort.NewLine);

                    //終端文字列が有ったとき
                    if (index > -1)
                    {
                        //終端文字列の削除
                        string rcvDataReturn = rcvData.Remove(index);

                        return rcvDataReturn;
                    }
                }
                catch (Exception ex)
                {
                    LabelState.Text = "受信エラー発生";
                    MessageBox.Show(ex.Message);
                    return null;
                }
            }
        }

        ///<summary>
        ///送受信
        ///</summary>
        ///<param name="cmd">送信文字列</param>
        ///<returns>受信文字列</returns>
        public string SendReceiveString(string cmd)
        {
            string data = null;

            if (SerialPort.IsOpen)
            {
                //送信
                SendString(cmd);

                System.Threading.Thread.Sleep(100);

                //受信
                data = ReceiveString();
            }

            return data;
        }


        ///<summary>
        ///ポジション設定ボタンを押した時の処理
        ///</summary>
        private void ButtonSetPosition_Click(object sender, EventArgs e)
        {
            //---------------------------------------------------------
            //現在位置設定
            //---------------------------------------------------------
            SendString("AXI" + AxisNo + ":POS " + TextBoxPosition.Text);
        }

        ///<summary>
        ///閉じるボタンを押した時の処理
        ///</summary>
        private void ButtonClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        ///<summary>
        ///終了時の処理
        ///</summary>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (SerialPort.IsOpen)
                SerialPort.Close();
        }
    }
}
